<?php

namespace FiloBlu\Refilo\Remote\Entity\Provider;

use FiloBlu\Refilo\Remote\Entity\EntityProviderInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;

/**
 * Class AbstractFromCollectionProvider
 * @package FiloBlu\Refilo\Remote\Entity\Provider
 */
abstract class AbstractFromCollectionProvider implements EntityProviderInterface
{
    /**
     * @var int
     */
    protected $pageSize = 500;
    /**
     * @var array
     */
    protected $items = [];
    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * AbstractFromCollectionProvider constructor.
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig
    ){
        $this->scopeConfig = $scopeConfig;
    }

    public function next()
    {
        return next($this->items);
    }

    /**
     * @return bool|float|int|string|null
     */
    public function key()
    {
        return current($this->items)->getId();
    }

    public function valid()
    {
        if ($this->getCollection()->getCurPage() < $this->getCollection()->getLastPageNumber()) {
            if (current($this->items) !== false) {
                return true;
            }

            $this->items = $this->getCollection()
                ->clear()
                ->setCurPage($this->getCollection()->getCurPage() + 1)
                ->getItems();
            return true;
        }

        return current($this->items) !== false;
    }

    abstract public function getCollection();

    /**
     *
     */
    public function rewind()
    {
        $pageSize = (int) $this->scopeConfig->getValue('sync/data/index_bulk_size');
        $this->pageSize = ($pageSize) ? $pageSize : $this->pageSize;

        $this->items = $this->getCollection()
            ->clear()
            ->setPageSize($this->pageSize)
            ->setCurPage(1)
            ->getItems();
    }
}
